# frozen_string_literal: true

require 'spec_helper_acceptance'
require 'pry'

describe 'manage ssh keys', :unless => UNSUPPORTED_PLATFORMS.include?(fact('osfamily')) do
  context 'ensure keys' do
    let(:pp) do
      <<-EOS
      class {'accounts':
        users => {
          'root' => {
            'purge_ssh_keys' => true,
            'ssh_keys' => {
              'key1' => {
                'type' => 'ssh-rsa',
                'key' => 'AAAA'
              },
              'key2' => {
                'type' => 'ssh-dss',
                'key' => 'BBBB'
              },
            }
          },
        }
      }
      EOS
    end

    it 'install user\'s keys' do
      shell 'echo "ssh-rsa CCCC key3" > /root/.ssh/authorized_keys'
      expect(apply_manifest(pp,
        :catch_failures => false,
        :debug => false).exit_code).to be_zero
    end

    describe file('/root/.ssh/authorized_keys') do
      it { is_expected.to be_file }
      it { is_expected.to be_readable.by('owner') }
      it { is_expected.not_to be_readable.by('group') }
      it { is_expected.not_to be_readable.by('others') }
    end

    describe command('cat /root/.ssh/authorized_keys | grep key1') do
      its(:exit_status) { is_expected.to eq 0 }
      its(:stdout) { is_expected.to match /ssh-rsa AAAA key1/ }
    end

    describe command('cat /root/.ssh/authorized_keys | grep key2') do
      its(:exit_status) { is_expected.to eq 0 }
      its(:stdout) { is_expected.to match /ssh-dss BBBB key2/ }
    end

    # following key should be purged by puppet
    describe command('cat /root/.ssh/authorized_keys | grep key3') do
      its(:stdout) { is_expected.not_to match /ssh-rsa CCCC key3/ }
      its(:exit_status) { is_expected.to eq 1 }
    end
  end

  context 'use custom key location' do
    let(:yaml) do
<<-EOS
  classes:
    - '::accounts'
  accounts::user_defaults:
    ssh_dir_owner: 'root'
    ssh_dir_group: 'root'
    manage_ssh_dir: false
  accounts::users:
    user1:
      authorized_keys_file: '/etc/ssh/user1_authorized_keys'
      comment: "user1"
      ssh_keys:
        'local_key':
          type: "ssh-rsa"
          key: "AAAB"
    user2:
      authorized_keys_file: '/etc/ssh/user2_authorized_keys'
      comment: "user2"
      ssh_keys:
        'user2':
          type: "ssh-rsa"
          key: "AAAAB="
EOS
    end

        let(:pp) do
<<-EOS
  lookup('classes', {merge => unique}).include
EOS
    end

    it 'runs without cycle' do
      shell "echo \"#{yaml}\" > #{HIERA_PATH}/hieradata/common.yaml"

      expect(apply_manifest(pp,
        :catch_failures => false,
        :debug => false).exit_code).to be_zero
    end

    describe file('/etc/ssh/user1_authorized_keys') do
      it { is_expected.to be_file }
      it { is_expected.to be_readable.by('owner') }
      it { is_expected.not_to be_readable.by('group') }
      it { is_expected.not_to be_readable.by('others') }
    end

    describe file('/etc/ssh/user2_authorized_keys') do
      it { is_expected.to be_file }
      it { is_expected.to be_readable.by('owner') }
      it { is_expected.not_to be_readable.by('group') }
      it { is_expected.not_to be_readable.by('others') }
    end

    describe command('stat -c "%a %U %G" /etc/ssh/user1_authorized_keys') do
      its(:exit_status) { is_expected.to eq 0 }
      its(:stdout) { is_expected.to match /600 root root/ }
    end

    describe command('cat /etc/ssh/user1_authorized_keys') do
      its(:exit_status) { is_expected.to eq 0 }
      its(:stdout) { is_expected.to match /ssh-rsa AAAB/ }
    end

    describe command('cat /etc/ssh/user2_authorized_keys') do
      its(:exit_status) { is_expected.to eq 0 }
      its(:stdout) { is_expected.to match /ssh-rsa AAAAB= user2/ }
    end

    describe command('stat -c "%a %U %G" /etc/ssh/user2_authorized_keys') do
      its(:exit_status) { is_expected.to eq 0 }
      its(:stdout) { is_expected.to match /600 root root/ }
    end

    after(:all) do
      shell "rm #{HIERA_PATH}/hieradata/common.yaml"
    end
 end


  context 'provide custom ssh options' do
    let(:pp) do
<<-EOS
  lookup('classes', {merge => unique}).include
EOS
    end

    it 'runs without cycle' do
      # we use global yaml config
      expect(apply_manifest(pp,
        :catch_failures => false,
        :debug => false).exit_code).to be_zero
    end

    describe file('/home/george/.ssh') do
      it { is_expected.to be_directory }
      it { is_expected.to be_readable.by('owner') }
      it { is_expected.not_to be_readable.by('group') }
      it { is_expected.not_to be_readable.by('others') }
    end

    describe file('/home/george/.ssh/authorized_keys') do
      it { is_expected.to be_file }
      it { is_expected.to be_readable.by('owner') }
      it { is_expected.not_to be_readable.by('group') }
      it { is_expected.not_to be_readable.by('others') }
    end

    describe command('cat /home/george/.ssh/authorized_keys') do
      its(:exit_status) { is_expected.to eq 0 }
      its(:stdout) { is_expected.to match /from="\*.sales.example.net,!pc.sales.example.net",permitopen="192.0.2.1:80" ssh-dss AAAAB2...19Q== george@example.net/ }
    end
 end
end
